#ifndef CAMERA_H_
#define CAMERA_H_

#include "nv_math.h"
#include "nv_algebra.h"
#include "Ray.h"

class Camera
{
private:
	double fov; // store in radians, specify in degrees
	vec3 up;
	vec3 position;
	vec3 center;
	mat4 *perspectiveMatrix;

public:
	Camera();
	~Camera();
	// fov specified in degrees
	void setParameters( double posX = 0.0, double posY = 0.0, double posZ = 0.0, double centerX = 0.0, 
			    double centerY = 0.0, double centerZ = 0.0, double upX = 0.0, double upY = 0.0, 
			    double upZ = 0.0, double fov = 0.0 );
	inline vec3 getPosition() { return this->position; }
	void getEyeBasis( vec3 &u, vec3 &v, vec3 &w ); // modifies the vectors to hold the eye basis
	mat3 eyeTransform();
	inline double getFov() { return fov; }
	mat4 getPerspectiveMatrix( int width, int height, double near, double far );
	Ray getRay( unsigned int row, unsigned int col, unsigned int width, unsigned int height );
};

#endif

