#include "Cone.h"
#include <iostream>
#include <math.h>
#include <iostream>
using namespace std;

Cone::Cone( vec3 pos, double radius, double height, mat4 transformation, vec3 ambient, vector<Light> lights,
			   vec3 matDiffuse, vec3 matSpecular, vec3 matEmissive, double shininess, double reflective, double refractive) : 
SceneObject( transformation, ambient, lights, matDiffuse, matSpecular, matEmissive, shininess, reflective, refractive ),
pos( pos ), radius( radius ), height(height)
{
	Kr = 1.0;
	
#ifdef CREATION
	cerr << "Creating sphere with ID=" << this->id << endl;
	cerr << "\tpos: " << pos << endl;
	cerr << "\tradius: " << radius << endl;
	cerr << "\tlights count: " << lights.size() << endl;
	cerr << endl;
#endif
}

const double Cone::threshold = 0.001;

int Cone::solveQuadratic(double a, double b, double c, double *r0, double *r1)
{
	// compute discriminant
	double discrim = b*b - 4.f*a*c;
	if (discrim < 0) // no real roots, return
		return 0;
	// use the quadratic formula to find t0 and t1
	double q;
	if (b < 0) 
		q = -.5f*(b - sqrtf(discrim));
	else
		q = -.5f*(b + sqrtf(discrim));
	*r0 = q/a;
	*r1 = c/q;
	if (*r0 > *r1) { // swap
		double tmp = *r0;
		*r0 = *r1;
		*r1 = tmp;
	}
	return 1;
}

void Cone::intersect( Ray &ray )
{
	vec3 origin = inverseTransformation * ray.getEye4();
	vec3 delta = inverseTransformation * ray.getPath4();
	
	vec3 diff = origin - pos;
	
	double k = radius/height;
	double t0,t1;
	k = k*k;
	double a, b, c;
	a = delta.x*delta.x + delta.y*delta.y + k*delta.z*delta.z;
	b = 2 * (delta.x*diff.x + delta.y*diff.y - k*delta.z*(diff.z-height));
	c = diff.x*diff.x + diff.y*diff.y - k*(diff.z-height)*(diff.z-height);
	
	if (solveQuadratic(a, b, c, &t0, &t1))
	{
		double t = t0;
		if(t < 0)
			t = t1;
	
		if( t < threshold) return;
		
		double pz = diff.z + t*delta.z;
		
			
		vec3 hit = origin + t*delta;
		
		if(ray.updateHit(t,this) && ((pz >= 0.0) && (pz <= height)))
		{
			vec3 normal = hit - pos;
			normal.normalize();
			
			ray.setNormalAtHit( this->normalRemapMat * normal );
		}
	}
}


Cone & Cone::operator = ( const Cone & rhs )
{
	// members inherited from SceneObject
	this->vertices = rhs.vertices;
	this->lights = rhs.lights;
	this->material = rhs.material;
	this->transformation = rhs.transformation;
	this->ambientLight = rhs.ambientLight;
	
	this->pos = rhs.pos;
	this->radius = rhs.radius;
	this->height = rhs.height;
	
	return *this;
}