#ifndef LIGHT_H_
#define LIGHT_H_

#include "nv_math.h"
#include "nv_algebra.h"

class Light
{
protected:
	int type;
	double constantAttenuation;
	double linearAttenuation;
	double quadraticAttenuation;
	vec3 coords;
	vec3 color;

public:
	Light( double r, double g, double b, double x, double y, double z, int type,
	    	double constantAttenuation, double linearAttenuation, double quadraticAttenuation );

	inline vec3 getColor() { return color; }
	inline vec3 getCoords() { return coords; }

	inline vec3 valueAt( double distance ) 
	{ 
		return ( 1/ (constantAttenuation + linearAttenuation * distance 
			      + quadraticAttenuation * (distance * distance) ) ) * color; 
	}


	Light & operator = ( const Light & rhs );

	inline bool isPointLight() { return type == point; }
	inline bool isDirectionalLight() { return type == directional; }

	static const int point;
	static const int directional;

};

#endif

