#include "Material.h"

Material::Material() :
		diffuse( vec3( 0.5, 0.5, 0.5 ) ), specular( vec3( 0.5, 0.5, 0.5 ) ), emissive( vec3( 0.0, 0.0, 0.0 ) ),
		shininess( 0.2 ), reflective(0.0), refractive(0.0), refractiveIndex( 0.0 )
{}

/*
Material::Material( vec3 &diffuse, vec3 &specular, vec3 &emissive, double shininess ) :
		diffuse( diffuse ), specular( specular ), emissive( emissive ), shininess( shininess ), reflective(0.0), refractive(0.0)
{}
*/

Material::Material( vec3 &diffuse, vec3 &specular, vec3 &emissive, double shininess, double reflective, double refractive, double refractiveIndex ) :
		diffuse( diffuse ), 
		specular( specular ), 
		emissive( emissive ), 
		shininess( shininess ), 
		refractiveIndex( refractiveIndex )
{
	if (    reflective >= 0.0 && reflective <= 1.0 
	     && refractive >= 0.0 && refractive <= 1.0
	     && reflective + refractive <= 1.0 
	   )
	{
		this->reflective = reflective;
		this->refractive = refractive;
	}
	else
	{
		this->reflective = 0.0;
		this->refractive = 0.0;
		cerr << "Ignoring invalid reflective/refractive values creating material" << endl;
		cerr << "\t(both must be between 0 and 1 inclusive, and reflective + refractive <= 1)";
	}
}

/*
Material::Material( vec3 &diffuse, vec3 &specular, vec3 &emissive, double shininess, double reflective) :
diffuse( diffuse ), specular( specular ), emissive( emissive ), shininess( shininess ), reflective(reflective), refractive(0.0)
{}
*/

Material & Material::operator = ( const Material & rhs )
{
	this->diffuse = rhs.diffuse;
	this->specular = rhs.specular;
	this->emissive = rhs.emissive;
	this->shininess = rhs.shininess;
	this->reflective = rhs.reflective;
	this->refractive = rhs.refractive;

	return *this;
}

