#include "Ray.h"

int Ray::nextId = 1;

Ray::Ray() :
	eye( vec3( 0.0, 0.0, 0.0 ) ),
	path( vec3( 0.0, 0.0, 0.0 ) ),
	eye4( vec4( 0.0, 0.0, 0.0, 1.0 ) ),
	path4( vec4( 0.0, 0.0, 0.0, 0.0 ) ),
	id( nextId++ ),
	t( -1.0 ),
	normalAtHit( vec3( 0.0, 0.0, 0.0 ) ),
	hitObject( NULL )
{}

Ray::Ray( vec3 eye, vec3 path ) : 
		eye( eye ), 
		path( path ),
	       	eye4( vec4( eye.x, eye.y, eye.z, 1.0 ) ),
		path4( vec4( path.x, path.y, path.z, 0.0 ) ),	
		id( nextId++ ), 
		t( -1.0 ), 
		normalAtHit( vec3( 0.0, 0.0, 0.0 ) ),
		hitObject( NULL )
{}

bool Ray::updateHit( double t, SceneObject *obj )
{
	if (  ( this->t < 0 && t > 0 ) || ( this->t > 0 && t < this->t ) ) {
		this->t = t;
		hitObject = obj;
		return true;
	}

	return false;
}

void Ray::moveEyeAlongPath( double distance )
{
	vec3 direction = this->path;
	direction.normalize();

	eye += distance * direction;
}

void Ray::print( ostream & o ) const 
{
	o << "id=" << id << ", path=" << path << ", origin=" << eye;
}

ostream & operator << ( ostream & o, const Ray & r )
{
	r.print( o );
	return o;
}

