#ifndef RAY_H_
#define RAY_H_

#include "debug.h"

#include <iostream>
#include <cstdlib>
#include "nv_math.h"
#include "nv_algebra.h"
#include "SceneObject.h"

class Ray
{
private:
	vec3 eye;
	vec3 path;
	vec4 eye4;
	vec4 path4;
	int id;
	double t; // t at point of intersection of paramaterized form of eye + t * path
	vec3 normalAtHit;
	SceneObject *hitObject;

	static int nextId;

public:
	Ray();
	Ray( vec3 eye, vec3 path );
	inline int getId() { return id; }
	inline vec3 getEye() { return eye; }
	inline vec4 getEye4() { return eye4; }
	inline vec3 getPath() { return path; }
	inline vec4 getPath4() { return path4; }
	inline vec3 valueAt( double t ) { return eye + t * path; }
	inline vec3 valueAtHit() { return hitObject ? eye + t * path : vec3( -1.0, -1.0, -1.0 ) ; }

	inline void setEye( vec3 val ) 
	{
		eye = val;
		eye4.x = val.x;
		eye4.y = val.y;
		eye4.z = val.z;
		eye4.w = 1.0;
	}

	inline void setPath( vec3 val )
	{
		path = val;
		path4.x = val.x;
		path4.y = val.y;
		path4.z = val.z;
		path4.w = 0.0;
	}

	inline vec3 getNormalAtHit() { return normalAtHit; }
	inline void setNormalAtHit( vec3 value ) 
	{
	       	value.normalize();	
		normalAtHit = value; 
	}

	inline double tAtHit() { return t; }

	inline vec3 colorAtHit( vector<SceneObject*> otherObjects, vec3 viewerPos ) 
	{ 
		return hitObject ? hitObject->getColorAt( *this, otherObjects, viewerPos ) : vec3( 0.0, 0.0, 0.0 ); 
	}

	inline SceneObject * getHitObject() { return hitObject; }
	inline bool hitAnObject() { return hitObject != NULL; }

	void moveEyeAlongPath( double distance ); // distance in world coordinates

	void print( ostream & out ) const;

	/* call this on a successful hit, if hit is before previous one the new object and t
	 *   will replace the old values
	 *
	 * returns true if value was updated, false otherwise
	 */
	bool updateHit( double t, SceneObject *obj );
};

ostream & operator << ( ostream & o, const Ray & r );

#endif

