#ifndef SCENE_H_
#define SCENE_H_

#include "debug.h"

#include <fstream>
#include <vector>
#include "Camera.h"
#include "Ray.h"
#include "SceneObject.h"
#include "Vertex.h"
#include "Light.h"
#include "Triangle.h"
#include "Sphere.h"
#include "Cone.h"

using namespace std;

class Scene 
{
private:
	Camera camera;
	int width;
	int height;
	int maxRecursionDepth;
	char *outputFile;

	// vertices in the following 2 variables are IDed by their index
	vector<Vertex> allVertices; // references to these are held inside objects
	vector<Vertex> allVertexNormals; // all vertices with normals

	// for debugging
	vector< vector<Ray> > *riggedRays;

	vector<vec4> normals;

	vector<SceneObject*> objects;
	vector<Light> lights;
	vector< vector<vec3> > pixels; // the image data, row->col->rgb

	void doSize( vector<char*> tokens ); // overwrites all previous calls, erases pixel data
	void doMaxDepth( vector<char*> tokens );
	void doOutput( vector<char*> tokens );
	void doCamera( vector<char*> tokens );
	void doSphere( vector<char*> tokens );
	void doCone( vector<char*> tokens );
	void doVertex( vector<char*> tokens );
	void doVertexNormal( vector<char*> tokens );
	void doTriangle( vector<char*> tokens );
	void doTriangleNormal( vector<char*> tokens );
	void doDirectionalLight( vector<char*> tokens );
	void doPointLight( vector<char*> tokens );

	void initStateVars();

	vec3 recursiveTrace( Ray ray, int remainingBounces );

public:
	Scene();
	bool parse( fstream &descriptor );
	void renderImage();
	void saveImage();
};

#endif

