#include "Sphere.h"
#include <iostream>
#include <math.h>
#include <iostream>
using namespace std;

Sphere::Sphere( vec3 pos, double radius, mat4 transformation, vec3 ambient, vector<Light> lights,
	     vec3 matDiffuse, vec3 matSpecular, vec3 matEmissive, double shininess, double reflective, double refractive) : 
	SceneObject( transformation, ambient, lights, matDiffuse, matSpecular, matEmissive, shininess, reflective, refractive ),
	pos( pos ), radius( radius )
{
	Kr = 1.0;
}

const double Sphere::threshold = 0.001;


void Sphere::intersect( Ray &ray )
{
	vec3 origin = inverseTransformation * ray.getEye4();
	vec3 delta = inverseTransformation * ray.getPath4();

	double a, b, c, discriminant;
	a = dot( delta, delta );
	b = 2 * dot( delta, origin - pos );
	c = dot( origin - pos, origin - pos ) - radius * radius;

	discriminant = b * b - 4 * a * c;

	if ( discriminant >= 0.0 )
	{
		double temp1 = (-b + sqrt(discriminant))/(2*a);
		double temp2 = (-b - sqrt(discriminant))/(2*a);
		double t;
	       	
		if ( temp1 > threshold && temp2 > threshold )
			t = temp1 < temp2 ? temp1 : temp2;

		// if both negative, thresholding cuts it off
		else 
			t = temp1 < 0.0 ? temp2 : temp1;

		if ( t < threshold )  return;

		vec3 hit = origin + t * delta;

		if ( ray.updateHit( t, this ) )
		{
			vec3 normal = hit - pos;
			normal.normalize();

			ray.setNormalAtHit( this->normalRemapMat * normal );
		}
	}
}


Sphere & Sphere::operator = ( const Sphere & rhs )
{
	// members inherited from SceneObject
	this->vertices = rhs.vertices;
	this->lights = rhs.lights;
	this->material = rhs.material;
	this->transformation = rhs.transformation;
	this->ambientLight = rhs.ambientLight;

	this->pos = rhs.pos;
	this->radius = rhs.radius;

	return *this;
}

