#include <cmath>
#include <iostream>
#include "Triangle.h"

using namespace std;

Triangle::Triangle( Vertex v1, Vertex v2, Vertex v3, mat4 transformation, 
		    vec3 ambient, vector<Light> lights, vec3 matDiffuse, vec3 matSpecular, 
		    vec3 matEmissive, double shininess, double reflective, double refractive ) :
	SceneObject( transformation, ambient, lights, matDiffuse, matSpecular, matEmissive, shininess, reflective, refractive )
{
	Kr = 1.0;
	
	vertices.push_back( v1 );
	vertices.push_back( v2 );
	vertices.push_back( v3 );

	vec3 normalVertex1 = transformation * v1.coords;
	vec3 normalVertex2 = transformation * v2.coords;
	vec3 normalVertex3 = transformation * v3.coords;

	cross( normal, normalVertex2 - normalVertex1, normalVertex3 - normalVertex1 );
	normal.normalize();

	mat4 inverseTranspose;
	transpose( inverseTranspose, inverseTransformation );
	normal = inverseTranspose * normal;
}

void Triangle::setNormal( vec3 normal )
{
	this->normal = normal;
}

const double Triangle::threshold = 0.001;

void Triangle::intersect( Ray &ray )
{
	if ( vertices.size() != 3 ) {
		cerr << "Bad number of vertices for triangle in Triangle::intersect (" << vertices.size() << ")" << endl << "[";
		for ( unsigned int i = 0; i < vertices.size(); i++ )
			cerr << vertices[ i ].coords << " ";

		cerr << "]" << endl << endl;
		return; 
	}

	vec3 a = vertices[ 0 ].coords;
	vec3 b = vertices[ 1 ].coords;
	vec3 c = vertices[ 2 ].coords;

	const vec3 d = inverseTransformation * ray.getPath4();
	const vec3 e =  inverseTransformation * ray.getEye4();

	double temp1, temp2, temp3;
	temp1 = dot((e-a), normal);
	temp2 = dot(d, normal);

	double t = -1*temp1/temp2;

	vec3 x = e + t*d;

	vec3 cross1, cross2, cross3;

	cross(cross1, (b-a), (x-a));
	temp1 = dot(cross1,normal);

	cross(cross2, (c-b), (x-b));
	temp2 = dot(cross2,normal);

	cross(cross3, (a-c), (x-c));
	temp3 = dot(cross3,normal);
	
	if((temp1 >= 0) && (temp2 >= 0) && (temp3 >= 0) && t > Triangle::threshold )
		if ( ray.updateHit( t, this ) )
			ray.setNormalAtHit( normal );
}

